@@ -83,7 +83,8 @@ struct _Board {
 
   Vector placed_balls; /* temp for place_received_bubble() placed here to 
 			  avoid mallocation at each call and do not get into trouble due to parallelism */
-   
+
+  int fixup_count; /* Recompute all ball position every 100 animations */
 };
 
 enum { 
@@ -163,6 +164,7 @@ Board new_board( int period, RuleSet_t *ruleset, int mov_ceiling,
   board->period = period;
   board->nb_rising_bubbles = 0;
   board->placed_balls = vector_new( 10 );
+  board->fixup_count = 0;
 
   /* alert blinking light */
   board->alert = new_sprite( BOTTOM_LAYER, 30, alert_animation, 0, 0);
@@ -363,6 +365,20 @@ int animate_bubbles( Board board, int dt ) {
   int count;
   int transition_allowed = 1;
   double x, y, t;
+  if (board->fixup_count++ > 10) {
+    board->fixup_count = 0;
+    for (i=0; i<board->bubbles->size; i++) {
+       bubble = board->bubbles->element[0];
+       if (bubble->state == STUCK) {
+	  cell_center( board->array, bubble->cell, &x, &y );
+	  if (x != bubble->x || y != bubble->y) {
+	     warn("Fixup a ball!");
+	     set_bubble_position( bubble, x, y);
+	  }
+       }
+    }
+  }
+   
   /* 
      possible board state transitions:
      
@@ -460,16 +476,21 @@ int animate_bubbles( Board board, int dt ) {
 				      bubble->cell, 
 				      board->explosive_bubbles,
 				      board->exposion_dates )) >= 3 ) {
+	  float violence=count*10;
+	  float direction; /* direction of each ball */
+
+	  if (violence > 100) violence=100;
 	  
 	  for ( i = 0; i < board->explosive_bubbles->size; i++ ) {
+	     
 	    tmp = board->explosive_bubbles->element[i];
 	    /* simulate propagation */ 
 	    clock = (1 - board->exposion_dates->element[i])*PROPAGATION_TIME;
 	    set_bubble_state( tmp, EXPLODING, TOP_LAYER, clock);
-	    tmp->vx = EXPLODING_SPEED;
-//	    tmp->vx = (rnd(100)-50)*EXPLODING_SPEED/10;
-	    tmp->vy = EXPLODING_SPEED;
-	    // FIXME also explode those balls, not only the unhooked ones
+	     
+	    direction = ((float)rand())/((float)RAND_MAX+1.0) * -3.14159;
+	    tmp->vx = cos(direction)*violence*EXPLODING_SPEED;
+	    tmp->vy = sin(direction)*violence*EXPLODING_SPEED;
 	    board->array->cell[tmp->cell] = EMPTY_CELL;
 	    if ( i >= 3 ) /* immediately send extra bubbles to opponent */ 
 	      vector_push( board->output, tmp->color );
@@ -481,7 +502,9 @@ int animate_bubbles( Board board, int dt ) {
 	transition_allowed = 0;
 	drop_bubbles(board);
 	place_received_bubbles(board);
-	board->state = ( board->canon_empty )? WAITING_FOR_CANON:READY_TO_FIRE;
+	/* allow transition only if no malus ball comes (or after the last malus ball arrives) */
+	if ( board->nb_rising_bubbles == 0 ) 
+	   board->state = ( board->canon_empty )? WAITING_FOR_CANON:READY_TO_FIRE;
       }
       else
 	set_bubble_position( bubble, bubble->x, bubble->y);
@@ -497,7 +520,8 @@ int animate_bubbles( Board board, int dt ) {
       bubble->clock += dt;
       if ( bubble->clock <= 0 )
 	break;
-      if ( bubble->clock > get_bubble_animation_duration(bubble) ) {
+      if ( bubble->clock > get_bubble_animation_duration(bubble) &&
+	   get_bubble_animation_nb_frame(bubble) > 1 ) {
 	kill_bubble( board, bubble );
 	break;
       }
@@ -553,6 +577,7 @@ int animate_bubbles( Board board, int dt ) {
 	stick_bubble( board, bubble, bubble->cell);
 	set_bubble_state( bubble, STUCK, TOP_LAYER, 0 );
 	board->nb_rising_bubbles--;
+	 
 	/* if all bubbles have been received then unlock board */
 	if ( board->nb_rising_bubbles == 0 ) {
 	  board->state =
@@ -697,10 +722,8 @@ void drop_bubbles(Board board) {
 	 set_bubble_state( bubble, FALLING, TOP_LAYER, 0);
 	 /* Which direction this ball goes to */
 	 direction = ((float)rand())/((float)RAND_MAX+1.0) * -3.14159;
-	 bubble->vx = cos(direction)*violence;
-	 bubble->vy = sin(direction)*violence;
-	 bubble->vx *= EXPLODING_SPEED;
-	 bubble->vy *= EXPLODING_SPEED;
+	 bubble->vx = cos(direction)*violence*EXPLODING_SPEED;
+	 bubble->vy = sin(direction)*violence*EXPLODING_SPEED;
 	 board->array->cell[cell] = EMPTY_CELL;
       }
    }
@@ -710,6 +733,7 @@ void drop_bubbles(Board board) {
 void lower_board(Board board) {
   int i, color;
   Bubble bubble;
+
   cell_array_lower( board->array );
   for ( i = 0; i < board->bubbles->size; i++ ) {
     bubble = board->bubbles->element[i];
