@@ -398,7 +398,8 @@ static void animate_game( Game game, int dt ) {
     animate_board( game->board[i], dt );
     lost = lost || get_board_state( game->board[i] ) == LOST;
     redraw_sprite_pool( get_board_sprite_pool(game->board[i]), 
-			game->board_pixmap[i], game->board_bg[i] );
+			game->board_pixmap[i], game->board_bg[i], 
+			board_was_lowered( game->board[i] ) /* redraw everything if the board was lowered */ );
   }
   if (( game->multi_player )&&( game->state != OVER )&&( !lost )) {
     int given = 0;
