@@ -702,18 +702,19 @@ void init_gcs() {
 }
 
 void cleanup_graphics() {
-  int color;
+//  int color;
   delete_animation( alert_animation );
   delete_animation( canon_animation );
   delete_animation( countdown_animation );
   /* FIXME: cleanup animations properly ! 
     delete_frame( bubble_animation[0][DEAD]->element[1] );
-   */
+
   for ( color = 0; color < NB_COLORS; color++ ) {
     delete_animation( bubble_animation[color][STUCK] );
     delete_animation( bubble_animation[color][EXPLODING] );
     delete_animation( bubble_animation[color][DEAD] );
   }
+*/
   XFreeGC( display, dialog_gc );
   XFreeGC( display, dialog_text_gc );
   XFreeGC( display, menu_gc );
