@@ -26,6 +26,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 
+#include "setting.h"
 #include "utils.h"
 #include "loadpng.h"
 #include "rgba.h"
@@ -33,10 +34,14 @@
 extern Display *display;
 extern Visual *visual;
 extern Window root;
-extern int screen, depth;
+extern int screen;
+extern int depth;
 
-/* precision of integer computations */
-#define FRACBITS       ( 12 ) /* 32 = 8 + 12 + 12 ( 1 byte << 12 twice )*/
+/* 
+   precision of integer computations
+   FRACBITS = 12 because 32 = 8 + 12 + 12 ( 1 byte << 12 twice )
+*/
+#define FRACBITS       ( 12 )
 #define FRACONE        ( 1 << FRACBITS )
 #define FRACBITSX2     ( 2*FRACBITS )
 #define FRACMASK       ( FRACONE - 1 )
@@ -59,12 +64,15 @@ void delete_rgba_image( RgbaImage ri ) {
 }
 
 RgbaImage create_rgba_image_from_png_file( const char *filename ) {
+  int width;
+  int height;
+  int has_alpha;
   unsigned char * data;
-  int width, height, has_alpha;
   RgbaImage ri;
-  data = load_png_file( filename, &width, &height, &has_alpha);
+
+  data = load_png_file( filename, &width, &height, &has_alpha );
   if ( data == NULL )
-    return NULL;
+    fail( "cannot load PNG file %s.", filename );
   ri = (RgbaImage) xmalloc( sizeof(struct _RgbaImage) );
   ri->data = data;
   ri->width = width;
@@ -73,18 +81,32 @@ RgbaImage create_rgba_image_from_png_file( const char *filename ) {
   return ri;
 }
 
-RgbaImage scale_down_rgba_image( RgbaImage ri, double scalex, double scaley ) {
+RgbaImage scale_rgba_image( RgbaImage ri, double scalex, double scaley ) {
   RgbaImage ri2;  
-  long r, g, b, a; /* 32 bits */
-  unsigned char *dest, *src, *srcy;
-  int *xsample, *ysample, new_width, new_height; 
-  int nb_samples, bytes_per_line, i, x, y, xrange, yrange, rx, ry;
+  long r;
+  long g;
+  long b;
+  long a; /* 32 bits */
+  unsigned char *dest;
+  unsigned char *src;
+  unsigned char *srcy;
+  int *xsample;
+  int *ysample;
+  int new_width;
+  int new_height; 
+  int nb_samples;
+  int bytes_per_line;
+  int i;
+  int x;
+  int y;
+  int xrange;
+  int yrange;
+  int rx;
+  int ry;
 
-  new_width = (int) floor( scalex * ri->width );
-  new_height = (int) floor( scaley * ri->height );
-  if (( new_width > ri->width )||( new_width <= 0 )||
-      ( new_height > ri->height )||( new_height <= 0 ))
-    return NULL;
+  new_width = MAX((int) floor( scalex * ri->width ), 1);
+  new_height = MAX((int) floor( scaley * ri->height ), 1);
+  
   ri2 = new_rgba_image( new_width, new_height);
   ri2->has_alpha = ri->has_alpha;
   ri2->hotx = ri->hotx * new_width / ri->width;
@@ -92,7 +114,7 @@ RgbaImage scale_down_rgba_image( RgbaImage ri, double scalex, double scaley ) {
   xsample = (int *) xmalloc( (new_width+1) * sizeof(int));
   ysample = (int *) xmalloc( (new_height+1) * sizeof(int));
   bytes_per_line = (ri->width << 2);
-
+  
   /* precompute scale data */
   for ( i = 0; i <= new_width; i++ )
     xsample[i] = i*ri->width/new_width;
@@ -144,34 +166,67 @@ RgbaImage scale_down_rgba_image( RgbaImage ri, double scalex, double scaley ) {
 RgbaImage crop_rgba_image( RgbaImage ri, int x1, int y1, int x2, int y2 ) {
   RgbaImage ri2;
   unsigned char *src;
-  int row, new_width, new_height, new_bytes_per_line, bytes_per_line;
-  if (( x1 >= 0 )&&( y1 >= 0 )&&
-      ( x2 < ri->width )&&( y2 < ri->height )&&
-      ( x1 <= x2 )&&( y1 <= y2 )) {
-    new_width = x2 - x1 + 1;
-    new_height = y2 - y1 + 1;
-    ri2 = new_rgba_image( new_width, new_height );
-    ri2->has_alpha = ri->has_alpha; 
-    ri2->hotx = ri->hotx - x1;
-    ri2->hoty = ri->hoty - y1;
-    bytes_per_line = ( ri->width << 2 );
-    new_bytes_per_line = ( new_width << 2 );
-    src = ri->data + (x1 << 2) + y1*bytes_per_line;
-    for ( row = 0; row < new_height; row++ )
-      memcpy( ri2->data + row*new_bytes_per_line, 
-	      src + row*bytes_per_line, new_bytes_per_line );
-    return ri2;
-  }
-  return NULL;
+  int row;
+  int new_width;
+  int new_height;
+  int new_bytes_per_line;
+  int bytes_per_line;
+  
+  if (( x1 < 0 )||
+      ( y1 < 0 )||
+      ( x2 >= ri->width )||
+      ( y2 >= ri->height )||
+      ( x1 > x2 )||
+      ( y1 > y2 ))
+    fail( "invalid cropping parameters !" );
+  
+  new_width = x2 - x1 + 1;
+  new_height = y2 - y1 + 1;
+  ri2 = new_rgba_image( new_width, new_height );
+  ri2->has_alpha = ri->has_alpha; 
+  ri2->hotx = ri->hotx - x1;
+  ri2->hoty = ri->hoty - y1;
+  bytes_per_line = ( ri->width << 2 );
+  new_bytes_per_line = ( new_width << 2 );
+  src = ri->data + (x1 << 2) + y1*bytes_per_line;
+  for ( row = 0; row < new_height; row++ )
+    memcpy( ri2->data + row*new_bytes_per_line, 
+	    src + row*bytes_per_line, new_bytes_per_line );
+  return ri2;
 }
 
 RgbaImage rotate_rgba_image( RgbaImage ri, double angle, int autocrop ) {
-  RgbaImage ri2, ri3;
-  int bytes_per_line, i, x, y, x1, y1, x2, y2; 
-  long ucos, usin, uxmax, uymax, sxfrac, syfrac; /* 32 bits */
-  long u, l, ul, ur, ll, lr, sx, sy, kx, ky; /* 32 bits */
-  unsigned char *src, *dest, *data, v;
+  int i;
+  int x;
+  int y;
+  int x1;
+  int y1;
+  int x2;
+  int y2; 
+  long ucos;
+  long usin;
+  long uxmax;
+  long uymax;
+  long sxfrac;
+  long syfrac;
+  long u;
+  long l;
+  long ul;
+  long ur;
+  long ll;
+  long lr;
+  long sx;
+  long sy;
+  long kx;
+  long ky;
+  unsigned char *src;
+  unsigned char *dest;
+  unsigned char *data;
+  unsigned char v;
+  int bytes_per_line;
   int width = ri->width;
+  RgbaImage ri2;
+  RgbaImage ri3;
 
   bytes_per_line = ( width << 2 );
   ri2 = new_rgba_image( ri->width, ri->height );
@@ -190,7 +245,7 @@ RgbaImage rotate_rgba_image( RgbaImage ri, double angle, int autocrop ) {
   y1 = ri->height;
   x2 = -1;
   y2 = -1;
-
+  
   for ( y = 0; y < ri->height; y++ ) {
     sx = kx - usin*y;
     sy = ky + ucos*y;
@@ -279,38 +334,44 @@ static void create_pixmap_from_image( XImage *xim, Pixmap *pixmap ) {
   gcv.background = 0;
   *pixmap = XCreatePixmap( display, root, xim->width, xim->height, xim->depth);
   gc = XCreateGC( display, *pixmap, GCForeground | GCBackground, &gcv);
-  XPutImage( display, *pixmap, gc, xim, 0,0,0,0, xim->width, xim->height);
+  XPutImage( display, *pixmap, gc, xim, 0, 0, 0, 0, xim->width, xim->height);
   XFreeGC( display, gc);
 }
 
-int create_pixmaps_from_rgba_image( RgbaImage ri, Pixmap *rgb, Pixmap *mask) {
+void create_pixmaps_from_rgba_image( RgbaImage ri, Pixmap *rgb, Pixmap *mask) {
   unsigned char *src;
-  unsigned long r, g, b, pixel; /* 32 bits */
-  int x, y, bitmap_pad; 
-  int lr_shift, lg_shift, lb_shift, rr_shift, rg_shift, rb_shift;
-  XImage *rgb_image, *mask_image;
-  if ( depth < 8 )
-    return 0;
+  unsigned long r;
+  unsigned long g;
+  unsigned long b;
+  unsigned long pixel;
+  int x;
+  int y;
+  int bitmap_pad; 
+  int lr_shift;
+  int lg_shift;
+  int lb_shift;
+  int rr_shift;
+  int rg_shift;
+  int rb_shift;
+  XImage *rgb_image;
+  XImage *mask_image;
 
+  if ( depth < 8 )
+    fail( "your display depth must be >= 8." );
+  
   bitmap_pad = ( depth > 16 )? 32 : (( depth > 8 )? 16 : 8 );
   /* create XImages */
   rgb_image = XCreateImage( display, visual, depth, ZPixmap, 0, NULL,
   			    ri->width, ri->height, bitmap_pad, 0);
   mask_image = XCreateImage( display, visual, 1, ZPixmap, 0, NULL,
 			     ri->width, ri->height, 8, 0);
-  if (( rgb_image == NULL )||( mask_image == NULL )) {
-    XDestroyImage(rgb_image);
-    XDestroyImage(mask_image);
-    return 0;
-  }
+  if (( rgb_image == NULL )||( mask_image == NULL ))
+    fail( "not enough memory for image allocation !" );
+  
   /* allocate data using bytes_per_line field */
-  rgb_image->data = (char *) malloc( rgb_image->bytes_per_line*ri->height );
-  mask_image->data = (char *) malloc( mask_image->bytes_per_line*ri->height );
-  if (( rgb_image->data == NULL )||( mask_image->data == NULL )) {
-    XDestroyImage(rgb_image);
-    XDestroyImage(mask_image);
-    return 0;
-  }
+  rgb_image->data =  (char *) xmalloc( rgb_image->bytes_per_line*ri->height );
+  mask_image->data = (char *) xmalloc( mask_image->bytes_per_line*ri->height );
+
   if ( depth > 8 ) {
     /* compute shifts for every channel */
     lr_shift = lsb( visual->red_mask );
@@ -346,17 +407,22 @@ int create_pixmaps_from_rgba_image( RgbaImage ri, Pixmap *rgb, Pixmap *mask) {
       else
 	XPutPixel( mask_image, x, y, 0);
     }
-  create_pixmap_from_image(rgb_image, rgb);
+  create_pixmap_from_image( rgb_image, rgb );
   if ( mask != NULL )
-    create_pixmap_from_image(mask_image, mask);
+    create_pixmap_from_image( mask_image, mask );
   /* cleanup */
   XDestroyImage(rgb_image);
   XDestroyImage(mask_image);
-  return 1;
 }
 
 unsigned long get_pixel( int red, int green, int blue ) {
-  int lr_shift, lg_shift, lb_shift, rr_shift, rg_shift, rb_shift;
+  int lr_shift;
+  int lg_shift;
+  int lb_shift;
+  int rr_shift;
+  int rg_shift;
+  int rb_shift;
+
   if ( depth > 8 ) {
     /* compute shifts for every channel */
     lr_shift = lsb( visual->red_mask );
