@@ -23,86 +23,126 @@
 #include <math.h>
 #include <unistd.h>
 #include <sys/time.h>
-
-#if TIME_WITH_SYS_TIME
-#include <time.h>
-#endif
-
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+
+#include "setting.h"
+#include "gettext.h"
+
 /*
 #include <mcheck.h>
 */
-#include "data.h"
+#include "cell.h"
 #include "game.h"
+#include "timer.h"
 #include "screens.h"
 #include "init.h"
 
 Display *display;
 Visual *visual;
-Window root, win;
-int screen, depth;
+Window root;
+Window win;
+int screen;
+int depth;
 Colormap colormap;
 
-int scale, win_width, win_height, fps;
+int scale;
+int win_width;
+int win_height;
+int fps;
 unsigned long frame_duration;
 char *data_dir;
+char *theme;
 int colors[NB_CELLS];
+int nb_levels;
+int level[MAX_NB_LEVELS][NB_CELLS];
 
-#define DEFAULT_FPS        100
-#define DEFAULT_HANDICAP   7000
-#define DEFAULT_PERIOD     12
-#define level_handicap(X)  ( DEFAULT_HANDICAP + 3000*( NORMAL - (X)))
+#define level_handicap(X) ( DEFAULT_HANDICAP + 3000*( NORMAL - (X)))
 
 void usage() {
-  printf("Usage: xbubble [-d display] [-s scale] [-f framerate] ");
-  printf("[-r resourcepath]\n\n");
+  printf(_("Usage: xbubble [-d display] [-s scale] [-f framerate] [-r resourcepath] [-t theme]\n\n"));
   exit(1);
 }
 
-static void randomize_colors( int nb_rows ) {
+static void create_randome_level( int nb_rows ) {
   int i;
   /* preload bubbles */
   for ( i = 0; i < nb_rows*COLS; i++ )
-    colors[i] = rnd( NB_COLORS );
+    colors[i] = rnd( NB_COLORS ) + 1;
+  for ( i = nb_rows*COLS; i < NB_CELLS; i++ )
+    colors[i] = 0;
 }
 
-void play_single_game( int *handicap ) {
+void play_single_player_game( int *handicap ) {
+  int i;
+  int won = 1;
+  int again = 1;
   int score[2]  = { 0, 0 };
+  RuleSet_t ruleset=DEFAULT_RULE_SET;
+  enum GameResult result;
   Game game;
-  game = new_game( SINGLE_PLAYER, handicap, False, NULL, 1, score, NORMAL );
-  play_game(game);
-  delete_game( game, False );
+
+  ruleset.max_fire_delay = handicap[0];
+  for ( i = 0; again && i < nb_levels; won && i++ ) {    
+    game = new_game( SINGLE_PLAYER, &ruleset, level[i], i+1, 
+		     score, NORMAL );
+    
+    result = play_game(game);
+    won = ( result == PLAYER1_WON );
+    again = ( result != ABORTED );
+    delete_game( game, again && ( !won || ( i+1 < nb_levels )));
+  }
+  if ( won )
+    printf(_("CONGRATULATIONS !!!\n"));
 }
 
 void play_challenge_game( int *handicap ) {
-  int score[2]  = { 0, 0 };
+  enum GameResult result;
+  int level=1;
+  int score[2] = { 0, 0 };
+  RuleSet_t ruleset=DEFAULT_RULE_SET;
   Game game;
+
+  ruleset.max_fire_delay = handicap[0];
   /* preload bubbles */
-  randomize_colors(5);
-  game = new_game( SINGLE_PLAYER, handicap, True, colors, 1, score, NORMAL );
-  play_game(game);
-  delete_game( game, False );
+  do {
+     create_randome_level(5);
+     game = new_game( SINGLE_PLAYER, &ruleset, colors, level, score,
+		      NORMAL );
+     result=play_game(game);
+     if (result == PLAYER1_WON)
+       level++;
+     delete_game( game, False );
+  } while (result != ABORTED);
 }
 
 void play_match( enum GameMode mode, int *handicap, enum Level level ) {
+  int round;
+  int done;
+  int score[2];
   int games[2]  = { 0, 0 };
   int rounds[2] = { 0, 0 };
-  int round, not_done, score[2];
+  RuleSet_t ruleset[2]={ DEFAULT_RULE_SET, DEFAULT_RULE_SET };
   enum GameResult result;
   Game game;
+
+  ruleset[0].max_fire_delay = handicap[0];
+  ruleset[1].max_fire_delay = handicap[1];
+
   do {
     score[0] = 0;
     score[1] = 0;
-    not_done = 1;
-    for ( round = 1; not_done; round++ ) {
+    done = 0;
+    for ( round = 1; !done; round++ ) {
       /* preload bubbles */
-      randomize_colors(5);
-      game = new_game( mode, handicap, True, colors, round, score, level );
+      create_randome_level(5);
+      game = new_game( mode, ruleset, colors, round, score, level);
       result = play_game(game);
-      not_done = ((( score[0] + score[1] < 2 )||( score[0] == score[1] ))&&
-		  ( result != ABORTED ));
-      delete_game( game, not_done );
+      done = score[1]==2 || score[0]==2 || result == ABORTED;
+       //not_done= (!(((score[1]>=5)&&(score[1]-score[0]>1)) || 
+       // ((score[0]>=5)&&(score[0]-score[1]>1))) &&
+       // ( result != ABORTED ));
+      delete_game( game, !done );
     }
     /* update stats */
     rounds[0] += score[0];
@@ -112,44 +152,61 @@ void play_match( enum GameMode mode, int *handicap, enum Level level ) {
       games[1] += ( score[1] > score[0] )? 1 : 0 ;
     }
   }
-  while ( screen3( mode, rounds, games ) == S3_CONTINUE );
+  while ( screen_2p( mode, rounds, games ) == S3_CONTINUE );
+}
+
+void play_demo( enum Level level ) {
+  int not_done;
+  RuleSet_t ruleset[2]={ DEFAULT_RULE_SET, DEFAULT_RULE_SET };
+  int score[2] = { 0, 0 };
+  Game game;
+  
+  ruleset[0].max_fire_delay = level_handicap(NORMAL);
+  ruleset[1].max_fire_delay = level_handicap(NORMAL);
+  
+  do {
+    create_randome_level(5);
+    game = new_game( DEMO, ruleset,  colors, 1, score, level );
+    not_done = ( play_game(game) != ABORTED );
+    delete_game( game, not_done );
+  }
+  while ( not_done );
 }
 
 void play_xbubble() {
-  int i, quit, level;
-  int choice[2], handicap[2];
-
-  /* empty preloaded bubbles array */
-  for ( i = 0; i < NB_CELLS; i++ )
-    colors[i] = -1; /* empty */
+  int quit, level;
+  int choice[2];
+  int handicap[2];
   quit = 0;
 
   while ( ! quit ) {
     handicap[0] = level_handicap(NORMAL);
     handicap[1] = level_handicap(NORMAL);
-    switch ( screen1() ) {
+
+    switch ( screen_main() ) {
 
     case S1_SINGLE_PLAYER:
       /* select handicap */
-      choice[0] = screen5();
+      choice[0] = screen_1p_handicap();
       if ( choice[0] != S5_ESCAPE ) {
 	handicap[0] = level_handicap( choice[0] );
-	play_single_game( handicap );
+	play_single_player_game( handicap );
       }
       break;
 
     case S1_TWO_PLAYERS:
       /* select players handicaps */
-      if ( screen4(choice) == S4_OK ) {
-	handicap[0] = level_handicap( choice[0] );
-	handicap[1] = level_handicap( choice[1] );
+      if ( screen_2p_handicap(choice) == S4_OK ) {
+	/* player 1 is on the left */
+	handicap[0] = level_handicap( choice[1] );
+	handicap[1] = level_handicap( choice[0] );
 	play_match( TWO_PLAYERS, handicap, NORMAL );
       }
       break;
 
     case S1_PLAYER_VS_COMPUTER:
       /* select computer skill */
-      level = screen2();
+      level = screen_computer_skill();
       if ( level != S2_ESCAPE ) {
 	level += VERY_EASY - S2_VERY_EASY;
 	play_match( PLAYER_VS_COMPUTER, handicap, level );
@@ -158,13 +215,18 @@ void play_xbubble() {
 
     case S1_CHALLENGE:
       /* select handicap */
-      choice[0] = screen5();
+      choice[0] = screen_1p_handicap();
       if ( choice[0] != S5_ESCAPE ) {
 	handicap[0] = level_handicap( choice[0] );
 	play_challenge_game( handicap );
       }
       break;
 
+    case S1_DEMO:
+      play_demo( VERY_HARD );
+      break;
+
+    case S1_ESCAPE:
     case S1_QUIT:
       quit = 1;
       break;
@@ -175,41 +237,53 @@ void play_xbubble() {
 }
 
 int main( int argc, char ** argv ) {
-  double zoom;
   int c;
-  char *display_name, app_name[128];
-  unsigned long timer_resolution;
-  int max_win_width, pscale;
+  double zoom;
+  char *display_name;
+  char app_name[256];
+  int max_win_width;
+  int pscale;
   struct timeval tv;
   XSizeHints xsh;
   XWMHints wm_hint;
   XClassHint class_hint;
 
+  /* NLS nuisances */
+#if ENABLE_NLS
+  setlocale(LC_ALL, "");
+  bindtextdomain(PACKAGE, LOCALEDIR);
+  textdomain(PACKAGE); 
+#endif
+   
   /* default values */
   fps = DEFAULT_FPS;
   data_dir = DATADIR;
+  theme = "fancy";
   pscale = 0;
   display_name = NULL;
   /* mtrace(); */
 
   /* parse options */
-  while (( c = getopt( argc, argv, "s:f:r:d:")) != -1 ) {
+  while (( c = getopt( argc, argv, "s:f:r:t:d:" )) != -1 ) {
     switch (c) {
     case 's': /* scale */
       if ( sscanf( optarg, "%d", &pscale ) != 1 )  {
-	fprintf(stderr, "Invalid scale: %s.\n", optarg);
+	fprintf( stderr, _("Invalid scale: %s.\n"), optarg);
 	usage();
       }
       break;
     case 'f': /* frame rate */
       if ( sscanf( optarg, "%d", &fps ) != 1 )  {
-	fprintf(stderr, "Invalid frame rate: %s.\n", optarg);
+	fprintf( stderr, _("Invalid frame rate: %s.\n"), optarg);
 	usage();
       }
       break;
     case 'r': /* resources */
       data_dir = optarg;
       break;
+    case 't': /* themes */
+      theme = optarg;
+      break;
     case 'd': /* display */
       display_name = optarg;
       break;
@@ -218,29 +292,32 @@ int main( int argc, char ** argv ) {
     }
   }
   init_display( display_name );
-  /* XSynchronize( display, True ); */
+
   /* adjust frame rate */
   fps = clip( fps, 10, 100 );
-  frame_duration = 1000000 / fps;
-  timer_resolution = 1000000 / CLK_TCK;
-  if ( frame_duration % timer_resolution )
-    frame_duration = ((frame_duration/timer_resolution)+1) * timer_resolution;
+  frame_duration = get_closest_itimer_interval(1000000/fps);
   fps = 1000000 / frame_duration;
 
   /* adjust scale */
   max_win_width = game_win_width(MAX_SCALE);
-  if ( ! pscale ) /* automagically adjust window size to 75% of screen width */
-    pscale = 75*WidthOfScreen(DefaultScreenOfDisplay(display))/max_win_width;
+  if ( ! pscale ) /* automagically adjust window size to 100% of screen width */
+    pscale = DEFAULT_SCALE*WidthOfScreen(DefaultScreenOfDisplay(display))/max_win_width;
   pscale = clip( pscale, 5, 100 );
   scale = pscale*MAX_SCALE/100;
   zoom = 1.0*scale/MAX_SCALE;
+
   win_width = game_win_width(scale);
   win_height = game_win_height(scale);
 
   /* create main window */
   win = XCreateSimpleWindow( display, root, 0, 0, win_width, win_height,0,0,0);
-  XSelectInput( display, win,
-		ExposureMask|StructureNotifyMask|KeyPressMask|KeyReleaseMask );
+  XSelectInput( display,
+		win,
+		ExposureMask |
+		StructureNotifyMask | 
+		KeyPressMask |
+		KeyReleaseMask );
+
   if ( depth == 8 )
     XSetWindowColormap( display, win, colormap );
   /* we don't want window resizing */
@@ -250,7 +327,7 @@ int main( int argc, char ** argv ) {
   XSetWMNormalHints( display, win, &xsh );
 
   /* set window title name, name, class, etc. */
-  sprintf( app_name, "XBubble @ %d fps  v%s", fps, VERSION );
+  sprintf( app_name, _("XBubble @ %d fps  v%s"), fps, VERSION );
   XStoreName( display, win, app_name );
   class_hint.res_name = "xbubble";
   class_hint.res_class = "XBubble";
@@ -272,5 +349,5 @@ int main( int argc, char ** argv ) {
   if ( depth == 8 )
     XFreeColormap( display, colormap );
   XCloseDisplay( display );
-  exit(0);
+  return 0;
 }
